/**
 * @file		RSETexture.c
 * @date		28/08/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		HMT texture parser and export to PNG format.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if defined(_WIN32)
	#include <windows.h>
#else
	#include <sys/types.h>
	#include <sys/stat.h>
#endif
#include "options.h"
#include <RSPTexture.h>
#include <RSPTexture_errordefs.h>
#include "texture_export.h"


/*
 * Internal functions declarations
 */

static unsigned int mainProcess(int args_cnt, char* args_value[], T_PROG_OPTIONS* opt_ptr);
static void createSubDir(char *dirName);
static unsigned short checkInputArgs(T_PROG_OPTIONS* opt_ptr, int p_arg_nbr, char* p_args[]);
static void dispHelp();


/*
 * - MAIN -
 */
int main(int argc, char *argv[]) {
	T_PROG_OPTIONS _opts;
	unsigned char p;

	// Hello world!
	printf("\n*~[ Rogue Squadron Extractor (RSE) - RSPTextureLib v%s ]~*\n", RSPTexture_getVersion());

	// Check for arguments
	if (argc < 2) {
		printf("\n[ERR] No input file/commands specified!\n");
		dispHelp();
		return RSPLIB_ERROR_ARGS_NULL;
	}

	// Create options for programs according to user's arguments.
	p = checkInputArgs(&_opts, argc, argv);
	if ( p == RSPLIB_ERROR_GENERIC ) return RSPLIB_SUCCESS;
	else if ( p != RSPLIB_SUCCESS ) return p;

	return mainProcess(argc, argv, &_opts);
}


/*
 * Private functions definition
 */

static unsigned int mainProcess(int args_cnt, char* args_value[], T_PROG_OPTIONS* p_opts) {
	unsigned short file_index;
	RSPTEXTURE_PARAMETERS libParams;
	T_RSPTEXTURE_HMT* hmtStruct = NULL;
	unsigned int i;

	libParams.raw = p_opts->raw & 0x7;

	// Manage multiple inputs files
	for ( file_index = p_opts->input_files_cnt; file_index < args_cnt; file_index++ )
	{
		printf("\n=============================================\n[INFO] - Parsing file: %s ...\n", args_value[file_index]);

		// Allocate T_RSPTEXTURE_HMT structure to store extracted datas.
		hmtStruct = calloc(1, sizeof(T_RSPTEXTURE_HMT));
		if (hmtStruct) {
			// Parse data from HMT file and put in T_RSPTEXTURE_HMT structure.
			if (RSPTexture_processHMTFile(hmtStruct, args_value[file_index], libParams) != RSPLIB_SUCCESS) {
				printf("[ERR] Failed to parse datas from %s\n", args_value[file_index]);
				RSPTexture_freeHMT(hmtStruct);
				return RSPLIB_ERROR_PROCESS;
			}
		} else return RSPLIB_ERROR_MEMORY;

		// Create output folders structure.
		if (p_opts->output_dir) createSubDir(args_value[file_index]);

		// Export each texture as a PNG picture.
		for ( i = 0; i < hmtStruct->textures_count; i++ ) {
			switch (hmtStruct->textures[i].type) {
			case 0:
			case 1:
			case 3:
			case 4:
			case 5:
				if (exportPNGTexture(&(hmtStruct->textures[i]), args_value[file_index], p_opts) == RSPLIB_SUCCESS)
					printf("[INFO] Successfully exported texture %s to PNG format.\n", hmtStruct->textures[i].name);
				else
					printf("[ERR] Failed to export texture %s to PNG format!\n", hmtStruct->textures[i].name);
				break;
			default:
				printf("[WARN] Can't export %s ! Image type %d not currently supported!\n", hmtStruct->textures[i].name, hmtStruct->textures[i].type);
			}
		}

		RSPTexture_freeHMT(hmtStruct);
	}

	return RSPLIB_SUCCESS;
}

static unsigned short checkInputArgs(T_PROG_OPTIONS* opt_ptr, int p_arg_nbr, char* p_args[]) {
	char test[256];
	int i;

	// Set default options
	opt_ptr->raw = 0;
	opt_ptr->output_dir = 1;

	if (p_arg_nbr > 1) {
		for ( i = 1; i < p_arg_nbr; i++) {
			strcpy(test, p_args[i]);
			if (p_args[i][0] != '-') break;
			if (strcmp(p_args[i], "-h") == 0) {
				dispHelp();
				return RSPLIB_ERROR_GENERIC;
			} else if (strcmp(p_args[i], "-v") == 0) {
				opt_ptr->verbose_mode = 1;
				printf("[OPTN] Verbose enabled.\n");
			} else if (strcmp(p_args[i], "-vv") == 0) {
				opt_ptr->verbose_mode = 1;
				opt_ptr->debug_mode = 1;
				printf("[OPTN] Debug enabled.\n");
			} else if (strcmp(p_args[i], "-vvv") == 0) {
				opt_ptr->verbose_mode = 1;
				opt_ptr->debug_mode = 1;
				opt_ptr->god_mode = 1;
				printf("[OPTN] God damn it!\n");
			} else if (strcmp(p_args[i], "-no-subdir") == 0) {
				opt_ptr->output_dir = 0;
				printf("[OPTN] Export to current directory.\n");
			} else {
				printf("[ERR] Unknown option: %s\n", p_args[i]);
			}
		}

		opt_ptr->input_files_cnt = i;
		return RSPLIB_SUCCESS;
	}

	return RSPLIB_ERROR_ARGS_NULL;
}

static void createSubDir(char *dirName) {
	if (dirName == NULL) return;
	char _dir[1024];

	snprintf(_dir, 1024, "%s-out", dirName);

	#ifdef _WIN32
		CreateDirectory(_dir, NULL);
	#else
		mkdir(_dir, 0755);
	#endif
}

static void dispHelp() {
	printf("\n");
	printf("Options:\n	-h  Print this message\n");
	printf("	-v -vv		Activate verbose console output\n");
	printf("	-no-subdir  Export models inside current folder\n");
	printf("\n");
	printf("Usage: RSETexture [options] <hmt_files...>\n");
	printf("\n");
}
