/**
 * @file		texture_export.c
 * @date		29/08/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		Export texture datas to PNG format.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <png.h>
#include "options.h"
#include <RSPTexture_datatypes.h>
#include <RSPTexture_errordefs.h>
#include "texture_export.h"


static inline T_R8G8B8A8 *getPixelAt(const T_RSPTEXTURE_TEXTURE *tex, const unsigned int posX , const unsigned int posY) {
	return tex->pixels + tex->width * posY + posX;
}

unsigned char exportPNGTexture(const T_RSPTEXTURE_TEXTURE* pTextureObj, const char* out_path, T_PROG_OPTIONS* p_opts) {
	char export_path[1024];
	FILE *_png_f = NULL;
	png_structp png_ptr = NULL;
	png_infop info_ptr = NULL;
	size_t x,y;
	png_byte **row_ptrs = NULL;
	T_R8G8B8A8 *pixel = NULL;
	//int pixel_size = 3;
	//int depth = 8;		//bit par color channel (RGB)

	if (pTextureObj == NULL || out_path == NULL)
		return RSPLIB_ERROR_ARGS_NULL;

#ifdef _WIN32
	if (p_opts->output_dir)
		snprintf(export_path, 1024, "%s-out\\%s.png", out_path, pTextureObj->name);
	else
		snprintf(export_path, 1024, "%s.png", pTextureObj->name);
#else
	if (p_opts->output_dir)
		snprintf(export_path, 1024, "%s-out/%s.png", out_path, pTextureObj->name);
	else
		snprintf(export_path, 1024, "%s.png", pTextureObj->name);
#endif

	// Open export file and add PNG header
	_png_f = fopen(export_path, "wb");
	if (_png_f == NULL) return RSPLIB_ERROR_MEMORY;
	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	if (png_ptr == NULL) {
		fclose(_png_f);
		return RSPLIB_ERROR_MEMORY;
	}
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL) {
		fclose(_png_f);
		return RSPLIB_ERROR_MEMORY;
	}

	// Set image attributes
	png_set_IHDR(png_ptr, info_ptr, pTextureObj->width, pTextureObj->height, 8, PNG_COLOR_TYPE_RGBA, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);

	// Store PNG datas in buffer
	row_ptrs = png_malloc(png_ptr, pTextureObj->height * sizeof(png_byte *));
	for ( y = 0; y < pTextureObj->height; y++ ) {
		png_byte *row = png_malloc(png_ptr, pTextureObj->width * sizeof(T_R8G8B8A8));
		row_ptrs[y] = row;
		for ( x = 0; x < pTextureObj->width; x++ ) {
			pixel = getPixelAt(pTextureObj, x , y);
			*row++ = pixel->r;
			*row++ = pixel->g;
			*row++ = pixel->b;
			*row++ = pixel->a;
		}
	}

	// Write pixels datas
	png_init_io(png_ptr, _png_f);
	png_set_rows(png_ptr, info_ptr, row_ptrs);
	png_write_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL);

	// Clean up
	for ( y = 0; y < pTextureObj->height; y++ ) {
		png_free(png_ptr, row_ptrs[y]);
	}
	png_free(png_ptr, row_ptrs);
	png_destroy_write_struct(&png_ptr, &info_ptr);

	fclose(_png_f);

	return RSPLIB_SUCCESS;
}
