/**
 * @file		hmt_parser.c
 * @date		26/08/2022
 * @author		JackCarterSmith
 * @copyright	GPL-v3.0
 * @brief		Process HMT file structure and extract its datas.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "RSPTexture_errordefs.h"
#include "RSPTexture_datatypes.h"
#include "hmt_struct.h"
#include "hmt_parser.h"


////////////////////////////////////////////////////////////////////////////////
//                       Private functions declarations
////////////////////////////////////////////////////////////////////////////////

static unsigned short ExtractTextureHMT(T_RSPTEXTURE_HMT*, const MEMFILE, const RSPTEXTURE_PARAMETERS*);
static unsigned short ExtractMaterials(T_RSPTEXTURE_HMT*, const MEMFILE, const RSPTEXTURE_PARAMETERS*);
static unsigned short ExtractTextures(T_RSPTEXTURE_HMT*, const MEMFILE, const RSPTEXTURE_PARAMETERS*);
static unsigned short TranslatePixels(T_RSPTEXTURE_TEXTURE*, const T_SAMPLE*, const T_R8G8B8*);
static unsigned short isTransparentColor(const T_R8G8B8A8, const T_R8G8B8A8);
static void convert4bitsGreyTo32bitsRGBA(T_R8G8B8A8*, const T_SAMPLE*, const unsigned int, const T_R8G8B8A8);
static void convert4bitsTo32bitsRGBA(T_R8G8B8A8*, const T_SAMPLE*, const unsigned int, const T_R8G8B8*, const T_R8G8B8A8);
static void convert8bitsTo32bitsRGBA(T_R8G8B8A8*, const T_SAMPLE*, const unsigned int, const T_R8G8B8*, const T_R8G8B8A8);
static void useOddBytes(T_R8G8B8A8*, const T_SAMPLE*, const unsigned int);


////////////////////////////////////////////////////////////////////////////////
//                         Public functions definition
////////////////////////////////////////////////////////////////////////////////

unsigned short RSP_TextureLib_ParseHMTMemFile(const MEMFILE pMemFile, T_RSPTEXTURE_HMT* hmtStruct, const RSPTEXTURE_PARAMETERS* pParams) {
	unsigned char err = RSPLIB_SUCCESS;

	if (hmtStruct != NULL && pMemFile != NULL) {
		// Do the magic!
		err = ExtractTextureHMT(hmtStruct, pMemFile, pParams);
	} else err = RSPLIB_ERROR_ARGS_NULL;

	return err;
}

unsigned short RSP_TextureLib_ParseHMTFile(const char* fileName, T_RSPTEXTURE_HMT* hmtStruct, const RSPTEXTURE_PARAMETERS* pParams) {
	unsigned char err = RSPLIB_SUCCESS;
	long fileSize;
	FILE* fStream = NULL;
	MEMFILE memFile = NULL;

	if (hmtStruct != NULL && fileName != NULL) {
		// Open file
		fStream = fopen(fileName, "rb");

		if (fStream != NULL) {
			// Determine file size in bytes
			fseek(fStream, 0, SEEK_END);
			fileSize = ftell(fStream);
			fseek(fStream, 0, SEEK_SET);
			if (pParams->verbose_mode) printf("[DBG] > Input file size: %ld bytes\n", fileSize);

			memFile = malloc(fileSize + 1);
			if (memFile != NULL) {
				// Copy file in RAM
				fread(memFile, fileSize, 1, fStream);
				fclose(fStream);

				// Do the magic!
				err = ExtractTextureHMT(hmtStruct, memFile, pParams);

				free(memFile);

			} else {
				fclose(fStream);
				err = RSPLIB_ERROR_MEMORY;
				if (pParams->verbose_mode) printf("[ERR] Can't allocate enough memory for file processing!\n");
			}
		} else {
			err = RSPLIB_ERROR_IO;
			if (pParams->verbose_mode) printf("[ERR] Input file %s not found!\n", fileName);
		}
	} else err = RSPLIB_ERROR_ARGS_NULL;

	return err;
}


////////////////////////////////////////////////////////////////////////////////
//                       Private functions definition
////////////////////////////////////////////////////////////////////////////////

/**
 * @brief Extract datas from texture HMT in memory.
 *
 * @param[out] pHmtStruct		Take T_RSPTEXTURE_HMT structure to fill with parsed datas.
 * @param[in] pMemfile			Pointer to an in-memory file location.
 * @param[in] pParams			Program option, used to tune parser features.
 *
 * @return Error code, RSPLIB_SUCCESS when no error.
 */
static unsigned short ExtractTextureHMT(T_RSPTEXTURE_HMT* pHmtStruct, const MEMFILE pMemfile, const RSPTEXTURE_PARAMETERS* pParams) {
	unsigned short errorCode = RSPLIB_SUCCESS;

	if (pHmtStruct == NULL || pMemfile == NULL) return RSPLIB_ERROR_ARGS_NULL;

	// Get global infos
	pHmtStruct->materials_count = ((T_HMTFILE_HEADER1 *)pMemfile)->materials_count;
	pHmtStruct->texture_offset = ((T_HMTFILE_HEADER1 *)pMemfile)->textures_offset;
	pHmtStruct->textures_count = ((T_HMTFILE_HEADER2 *)(pMemfile + pHmtStruct->texture_offset))->textures_count;

	printf("[INFO] > Materials count: %d\n", pHmtStruct->materials_count);
	printf("[INFO] > Textures count: %d\n", pHmtStruct->textures_count);
	if (pParams->verbose_mode) printf("[DBG] > Texture offset: 0x%X\n", pHmtStruct->texture_offset);
	printf("\n");

	// Get textures if any
	if (pHmtStruct->textures_count > 0)
		errorCode |= ExtractTextures(pHmtStruct, pMemfile, pParams);

	// Get materials if any and link them to texture
	if (pHmtStruct->materials_count > 0)
		errorCode |= ExtractMaterials(pHmtStruct, pMemfile, pParams);


	return errorCode;
}

/**
 * @brief Retrieve materials datas from HMT file.
 *
 * @param[in|out] pHmt		Take HMT structure to get the T_HMTFILE_MATERIAL buffer and header datas.
 * @param[in] pMemfile		Pointer to an in-memory file location.
 * @param[in] pParams		Program option, used to tune parser features.
 *
 * @return Error code, RSPLIB_SUCCESS when no error.
 */
static unsigned short ExtractMaterials(T_RSPTEXTURE_HMT* pHmt, const MEMFILE pMemfile, const RSPTEXTURE_PARAMETERS* pParams) {
	unsigned int i;

	if (pHmt == NULL || pMemfile == NULL) return RSPLIB_ERROR_ARGS_NULL;

	pHmt->materials = (T_RSPTEXTURE_MATERIAL *)calloc(pHmt->materials_count, sizeof(T_RSPTEXTURE_MATERIAL));
	if (pHmt->materials == NULL) return RSPLIB_ERROR_MEMORY;

	for ( i = 0; i < pHmt->materials_count; i++ ) {
		if (pParams->debug_mode) printf("\n-----------------------Begin of material #%d-------------------------\n", i);

		// Get material name
		//memcpy(pHmt->materials[i].name, ((T_HMTFILE_MATERIAL *)(pMemfile + sizeof(T_HMTFILE_MATERIAL) * (i + 1)))->name, 16);
		snprintf((char *)pHmt->materials[i].name, 16, "%s", ((T_HMTFILE_MATERIAL *)(pMemfile +
				sizeof(T_HMTFILE_HEADER1) + sizeof(T_HMTFILE_MATERIAL) * i))->name);

		// Get material datas
		pHmt->materials[i].type = ((T_HMTFILE_MATERIAL *)(pMemfile +
				sizeof(T_HMTFILE_HEADER1) + sizeof(T_HMTFILE_MATERIAL) * i))->type;
		pHmt->materials[i].id = ((T_HMTFILE_MATERIAL *)(pMemfile +
				sizeof(T_HMTFILE_HEADER1) + sizeof(T_HMTFILE_MATERIAL) * i))->texture_index;
		// If material is textured, the linked texture correspond to tex_index in textures array.
		if (pHmt->materials[i].type == RSPTEX_MAT_TYPE_TEXTURED) pHmt->materials[i].texture = pHmt->textures + pHmt->materials[i].id;
		else pHmt->materials[i].texture = NULL;
		pHmt->materials[i].opacity = ((T_HMTFILE_MATERIAL *)(pMemfile +
						sizeof(T_HMTFILE_HEADER1) + sizeof(T_HMTFILE_MATERIAL) * i))->reserved1;
		pHmt->materials[i].ambient = ((T_HMTFILE_MATERIAL *)(pMemfile +
								sizeof(T_HMTFILE_HEADER1) + sizeof(T_HMTFILE_MATERIAL) * i))->reserved0;

		if (pParams->verbose_mode) {
			printf("[INFO] > Material name: %s\n", pHmt->materials[i].name);
			printf("[INFO] > Material type: %d\n", pHmt->materials[i].type);
			printf("[INFO] > Material id: %d\n", pHmt->materials[i].id);
			printf("[INFO] > Material text. index: %d\n", i);
			if (pHmt->materials[i].type == RSPTEX_MAT_TYPE_TEXTURED)
				printf("[INFO] > Linked texture name: %s (should correspond to material name)\n", pHmt->materials[i].texture->name);
		}

		/*
		 * Unassigned generic material datas :
		 * - Transparent
		 * - Ambient (global)
		 * - Diffuse (environnement)
		 * - Specular (self)
		 */
		if (pParams->debug_mode) {
			printf("[DBG] > Material reserved0: %.8f\n", pHmt->materials[i].ambient);
			printf("[DBG] > Material reserved1: %.8f\n", pHmt->materials[i].opacity);
			printf("[DBG] > Material reserved2 (zero): 0x%X\n", ((T_HMTFILE_MATERIAL *)(pMemfile +
					sizeof(T_HMTFILE_HEADER1) + sizeof(T_HMTFILE_MATERIAL) * i))->reserved2);
			printf("[DBG] > Material reserved3 (0xA): 0x%X\n", ((T_HMTFILE_MATERIAL *)(pMemfile +
					sizeof(T_HMTFILE_HEADER1) + sizeof(T_HMTFILE_MATERIAL) * i))->reserved3);
		}

		if (pParams->debug_mode) printf("\n-----------------------End of material #%d---------------------------\n", i);
	}

	return RSPLIB_SUCCESS;
}

/**
 * @brief Retrieve textures datas from HMT file.
 *
 * @param[in|out] pHmt		Take HMT structure to get the T_HMTFILE_TEXTURE_HEADER buffer and header datas.
 * @param[in] pMemfile		Pointer to an in-memory file location.
 * @param[in] pParams		Program option, used to tune parser features.
 *
 * @return Error code, RSPLIB_SUCCESS when no error.
 */
static unsigned short ExtractTextures(T_RSPTEXTURE_HMT* pHmt, const MEMFILE pMemfile, const RSPTEXTURE_PARAMETERS* pParams) {
	T_R8G8B8* _palette = NULL;
	T_SAMPLE* _samples = NULL;
	unsigned int palette_offset, pixels_offset;
	unsigned int i;

	if (pHmt == NULL || pMemfile == NULL) return RSPLIB_ERROR_ARGS_NULL;

	pHmt->textures = (T_RSPTEXTURE_TEXTURE *)calloc(pHmt->textures_count, sizeof(T_RSPTEXTURE_TEXTURE));
	if (pHmt->textures == NULL) return RSPLIB_ERROR_MEMORY;

	for ( i = 0; i < pHmt->textures_count; i++ ) {
		if (pParams->debug_mode) printf("\n-----------------------Begin of texture #%d-------------------------\n", i);
		_palette = NULL;
		_samples = NULL;

		// Get texture size infos
		pHmt->textures[i].width = ((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset	+
				sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->tex_width;
		pHmt->textures[i].height = ((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset +
				sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->tex_height;
		pHmt->textures[i].type = ((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset +
				sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->tex_format.type;
		switch (pHmt->textures[i].type) {
		case 0:
			pHmt->textures[i].sample_bits = 4;
			pHmt->textures[i].palette_entries = 16;
			break;
		case 1:
			pHmt->textures[i].sample_bits = 8;
			pHmt->textures[i].palette_entries = 256;
			break;
		case 2:
			pHmt->textures[i].sample_bits = 16;
			pHmt->textures[i].palette_entries = 0;
			break;
		case 3:
			pHmt->textures[i].sample_bits = 32;
			pHmt->textures[i].palette_entries = 0;
			break;
		case 4:
			pHmt->textures[i].sample_bits = 4;
			pHmt->textures[i].palette_entries = 0;
			break;
		case 5:
			pHmt->textures[i].sample_bits = 8;
			pHmt->textures[i].palette_entries = 0;
			break;
		default:
			pHmt->textures[i].sample_bits = 0;
			pHmt->textures[i].palette_entries = 0;
		}
		memcpy(&pHmt->textures[i].alpha_color, &((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset +
				sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->tex_format.transparent_color, sizeof(T_R8G8B8A8));

		// Get texture datas offsets
		pixels_offset = ((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset +
				sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->pixels_offset;
		palette_offset = ((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset +
				sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->palette_offset;

		// Get texture name
		//memcpy(pHmt->textures[i].name, pMemfile + ((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset + sizeof(T_HMTFILE_TEXTURE_HEADER) * (i + 1)))->texname_offset, 16);
		snprintf((char *)pHmt->textures[i].name, 16, "%s", pMemfile + ((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset +
				sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->texname_offset);

		if (pParams->verbose_mode) {
			printf("[INFO] > Texture name: %s\n", pHmt->textures[i].name);
			printf("[INFO] > Texture size: %dx%d\n", pHmt->textures[i].width, pHmt->textures[i].height);
			printf("[INFO] > Texture type: %d\n", pHmt->textures[i].type);
			printf("[INFO] >   Samplebits: %d\n", pHmt->textures[i].sample_bits);
			printf("[INFO] >   Palette entries: %d\n", pHmt->textures[i].palette_entries);
			printf("[INFO] > Texture bits per sample: %d\n", ((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset + sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->tex_format.sample_bitsize);
			printf("[INFO] > Texture transparent color: %d %d %d %d\n", pHmt->textures[i].alpha_color.r, pHmt->textures[i].alpha_color.g, pHmt->textures[i].alpha_color.b, pHmt->textures[i].alpha_color.a);
		}
		if (pParams->debug_mode) {
			printf("[DBG] > Texture unknown0: 0x%X\n", ((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset + sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->tex_format.unknown0);
			if (((T_HMTFILE_TEXTURE_HEADER *)(pMemfile + pHmt->texture_offset + sizeof(T_HMTFILE_HEADER2) + sizeof(T_HMTFILE_TEXTURE_HEADER) * i))->tex_format.reserved0 != 1)
				printf("[DBG] > Texture: Always 1 different.\n");
			printf("[DBG] > Texture palette offset: 0x%X\n", palette_offset);
			printf("[DBG] > Texture pixels offset: 0x%X\n", pixels_offset);
		}

		// Retrieve palette datas from HMT file
		switch (pHmt->textures[i].palette_entries) {
		case 16:
		case 256:
			_palette = (T_R8G8B8 *)realloc(_palette, sizeof(T_R8G8B8) * pHmt->textures[i].palette_entries);
			if (_palette)
				memcpy(_palette, pMemfile + palette_offset, sizeof(T_R8G8B8) * pHmt->textures[i].palette_entries);
			else return RSPLIB_ERROR_MEMORY;
			break;
		default:
			break;
		}

		// Retrieve samples from HMT file
		//TODO: better approach?
		_samples = (T_SAMPLE *)realloc(_samples, pHmt->textures[i].width * pHmt->textures[i].height * pHmt->textures[i].sample_bits / 8);
		if (_samples) {
			memcpy(_samples, pMemfile + pixels_offset, pHmt->textures[i].width * pHmt->textures[i].height * pHmt->textures[i].sample_bits / 8);
			//if (pHmt->textures[i].type == 2) memcpy(pHmt->textures[i].samples, pMemfile + pHmt->textures[i].pixels_offset, pHmt->textures[i].width * pHmt->textures[i].height * pHmt->textures[i].sample_bits / (8 * 4));		//TODO: manage texture type 2
		} else return RSPLIB_ERROR_MEMORY;

		// Decode palette and samples to pixels datas
		TranslatePixels(&(pHmt->textures[i]), _samples, _palette);

		if (pParams->debug_mode) printf("\n-----------------------End of texture #%d---------------------------\n", i);
	}

	if (_palette) free(_palette);
	if (_samples) free(_samples);

	return RSPLIB_SUCCESS;
}

/**
 * @brief Convert samples to pixels using palette and texture informations.
 *
 * @param[in|out] pTexture	Take texture structure to get the pixels buffer and texture datas.
 * @param[in] pSamples		Pointer to samples buffer.
 * @param[in] pPalette		Pointer to palette buffer.
 *
 * @return Error code, RSPLIB_SUCCESS when no error.
 */
static unsigned short TranslatePixels(T_RSPTEXTURE_TEXTURE* pTexture, const T_SAMPLE* pSamples, const T_R8G8B8* pPalette) {
	unsigned int size;

	if (pTexture == NULL) return RSPLIB_ERROR_ARGS_NULL;
	if (!(pTexture->type == 0 ||
			pTexture->type == 1 ||
			pTexture->type == 2 ||
			pTexture->type == 3 ||
			pTexture->type == 4 ||
			pTexture->type == 5)) return RSPLIB_ERROR_PROCESS;

	size = pTexture->height * pTexture->width;

	switch (pTexture->sample_bits) {
	case 32:
		pTexture->pixels = (T_R8G8B8A8 *)calloc(size, sizeof(T_R8G8B8A8));
		if (pTexture->pixels == NULL) printf("[ERR] A texture failed to assert pixels buffer!");
		memcpy(pTexture->pixels, pSamples, size * sizeof(T_R8G8B8A8));
		break;
	case 4:
		pTexture->pixels = (T_R8G8B8A8 *)calloc(size, sizeof(T_R8G8B8A8));
		if (pTexture->pixels == NULL) printf("[ERR] A texture failed to assert pixels buffer!");
		if (pTexture->palette_entries == 0) {
			convert4bitsGreyTo32bitsRGBA(pTexture->pixels, pSamples, size, pTexture->alpha_color);
		} else if (pTexture->palette_entries == 16) {
			convert4bitsTo32bitsRGBA(pTexture->pixels, pSamples, size, pPalette, pTexture->alpha_color);
		}
		break;
	case 8:
		if (pTexture->palette_entries == 0) {
			pTexture->pixels = (T_R8G8B8A8 *)calloc(size, 1);
			if (pTexture->pixels == NULL) printf("[ERR] A texture failed to assert pixels buffer!");
			memcpy(pTexture->pixels, pSamples, size);
		} else if (pTexture->palette_entries == 256) {
			pTexture->pixels = (T_R8G8B8A8 *)calloc(size, sizeof(T_R8G8B8A8));
			if (pTexture->pixels == NULL) printf("[ERR] A texture failed to assert pixels buffer!");
			convert8bitsTo32bitsRGBA(pTexture->pixels, pSamples, size, pPalette, pTexture->alpha_color);
		}
		break;
	case 16:
		pTexture->pixels = (T_R8G8B8A8 *)calloc(size, 1);
		if (pTexture->pixels == NULL) printf("[ERR] A texture failed to assert pixels buffer!");
		useOddBytes(pTexture->pixels, pSamples, size);
		break;
	default:
		break;
	}

	return RSPLIB_SUCCESS;
}

static unsigned short isTransparentColor(const T_R8G8B8A8 testColor, const T_R8G8B8A8 transp_color) {
	if (!(testColor.r == transp_color.r)) return 0;
	if (!(testColor.g == transp_color.g)) return 0;
	if (!(testColor.b == transp_color.b)) return 0;

	return 1;
}

static void convert4bitsGreyTo32bitsRGBA(T_R8G8B8A8* pPixels, const T_SAMPLE* pSamples, const unsigned int sampling, const T_R8G8B8A8 transp_color) {
	unsigned int i;
	T_SAMPLE p;

	for ( i = 0; i < (sampling / 2); i++ ) {
		p = pSamples[i];

		pPixels[i * 2].r = div((p >> 4 & 0xF) * 256, 16).quot;
		pPixels[i * 2].g = div((p >> 4 & 0xF) * 256, 16).quot;
		pPixels[i * 2].b = div((p >> 4 & 0xF) * 256, 16).quot;
		pPixels[i * 2].a = isTransparentColor(pPixels[i * 2], transp_color) ? 255 - transp_color.a : 255;
		pPixels[i * 2 + 1].r = div((p & 0xF) * 256, 16).quot;
		pPixels[i * 2 + 1].g = div((p & 0xF) * 256, 16).quot;
		pPixels[i * 2 + 1].b = div((p & 0xF) * 256, 16).quot;
		pPixels[i * 2 + 1].a = isTransparentColor(pPixels[i * 2 + 1], transp_color) ? 255 - transp_color.a : 255;
	}
}

static void convert4bitsTo32bitsRGBA(T_R8G8B8A8* pPixels, const T_SAMPLE* pSamples, const unsigned int sampling, const T_R8G8B8* pal, const T_R8G8B8A8 transp_color) {
	unsigned int i;
	T_SAMPLE p;

	for ( i = 0; i < (sampling / 2); i++ ) {
		p = pSamples[i];

		pPixels[i * 2].r = pal[(p >> 4) & 0xF].r;
		pPixels[i * 2].g = pal[(p >> 4) & 0xF].g;
		pPixels[i * 2].b = pal[(p >> 4) & 0xF].b;
		pPixels[i * 2].a = isTransparentColor(pPixels[i * 2], transp_color) ? 255 - transp_color.a : 255;
		pPixels[i * 2 + 1].r = pal[p & 0xF].r;
		pPixels[i * 2 + 1].g = pal[p & 0xF].g;
		pPixels[i * 2 + 1].b = pal[p & 0xF].b;
		pPixels[i * 2 + 1].a = isTransparentColor(pPixels[i * 2 + 1], transp_color) ? 255 - transp_color.a : 255;
	}
}

static void convert8bitsTo32bitsRGBA(T_R8G8B8A8* pPixels, const T_SAMPLE* pSamples, const unsigned int sampling, const T_R8G8B8* pal, const T_R8G8B8A8 transp_color) {
	unsigned int i;
	T_SAMPLE p;

	for ( i = 0; i < sampling; i++ ) {
		p = pSamples[i];
		pPixels[i].r = pal[p].r;
		pPixels[i].g = pal[p].g;
		pPixels[i].b = pal[p].b;
		pPixels[i].a = isTransparentColor(pPixels[i], transp_color) ? 255 - transp_color.a : 255;
	}
}

static void useOddBytes(T_R8G8B8A8* pPixels, const T_SAMPLE* pSamples, const unsigned int sampling) {
	unsigned int i;

	for ( i = 0; i < (sampling - 1); i++ ) {
		//dst[i] = src[i*2+1];			//FIXME: Implement optimized byte shifting
		//dst[i]._red = src[i*2+1];
		//dst[i]._green = src[i*2+1];
		//dst[i]._blue = src[i*2+1];
	}
}
